@@ -84,7 +84,7 @@ fitMk<-function(tree,x,model="SYM",fixedQ=NULL,...){
 			el<-pw$edge.length[ii]
 			v<-vector(length=length(desc),mode="list")
 			for(j in 1:length(v))
-				v[[j]]<-expm(Q*el[j])%*%liks[desc[j],]
+				v[[j]]<-EXPM(Q*el[j])%*%liks[desc[j],]
 			vv<-if(anc==root) Reduce('*',v)[,1]*pi else Reduce('*',v)[,1]
 			comp[anc]<-sum(vv)
 			liks[anc,]<-vv/comp[anc]
@@ -154,7 +154,11 @@ summary.fitMk<-function(object,...){
 }
 
 ## logLik method for objects of class "fitMk"
-logLik.fitMk<-function(object,...) object$logLik
+logLik.fitMk<-function(object,...){ 
+	lik<-object$logLik
+	attr(lik,"df")<-length(object$rates)
+	lik
+}
 
 ## AIC method
 AIC.fitMk<-function(object,...,k=2){
@@ -250,3 +254,73 @@ plot.gfit<-function(x,...){
 		plot(obj,...)
 	}
 }
+
+## wraps around expm
+## written by Liam Revell 2011, 2017
+EXPM<-function(x,...){
+	e_x<-if(isSymmetric(x)) matexpo(x) else expm(x,...)
+	dimnames(e_x)<-dimnames(x)
+	e_x
+}
+
+## function to simulate multiple-rate Mk multiMk
+## written by Liam J. Revell 2018
+sim.multiMk<-function(tree,Q,anc=NULL,nsim=1,...){
+	if(hasArg(as.list)) as.list<-list(...)$as.list
+	else as.list<-FALSE
+	ss<-rownames(Q[[1]])
+	tt<-map.to.singleton(reorder(tree))
+	P<-vector(mode="list",length=nrow(tt$edge))
+	for(i in 1:nrow(tt$edge))
+		P[[i]]<-expm(Q[[names(tt$edge.length)[i]]]*tt$edge.length[i])
+	if(nsim>1) X<- if(as.list) vector(mode="list",length=nsim) else 
+		data.frame(row.names=tt$tip.label)
+	for(i in 1:nsim){
+		a<-if(is.null(anc)) sample(ss,1) else anc
+		STATES<-matrix(NA,nrow(tt$edge),2)
+		root<-Ntip(tt)+1
+		STATES[which(tt$edge[,1]==root),1]<-a
+		for(j in 1:nrow(tt$edge)){
+			new<-ss[which(rmultinom(1,1,P[[j]][STATES[j,1],])[,1]==1)]
+			STATES[j,2]<-new
+			ii<-which(tt$edge[,1]==tt$edge[j,2])
+			if(length(ii)>0) STATES[ii,1]<-new
+		}
+		x<-as.factor(
+			setNames(sapply(1:Ntip(tt),function(n,S,E) S[which(E==n)],
+			S=STATES[,2],E=tt$edge[,2]),tt$tip.label))
+		if(nsim>1) X[,i]<-x else X<-x
+	}
+	X
+}
+
+## constant-rate Mk model simulator
+## written by Liam J. Revell 2018
+sim.Mk<-function(tree,Q,anc=NULL,nsim=1,...){
+	if(hasArg(as.list)) as.list<-list(...)$as.list
+	else as.list<-FALSE
+	ss<-rownames(Q)
+	tt<-reorder(tree)
+	P<-vector(mode="list",length=nrow(tt$edge))
+	for(i in 1:nrow(tt$edge))
+		P[[i]]<-expm(Q*tt$edge.length[i])
+	if(nsim>1) X<- if(as.list) vector(mode="list",length=nsim) else 
+		data.frame(row.names=tt$tip.label)
+	for(i in 1:nsim){
+		a<-if(is.null(anc)) sample(ss,1) else anc
+		STATES<-matrix(NA,nrow(tt$edge),2)
+		root<-Ntip(tt)+1
+		STATES[which(tt$edge[,1]==root),1]<-a
+		for(j in 1:nrow(tt$edge)){
+			new<-ss[which(rmultinom(1,1,P[[j]][STATES[j,1],])[,1]==1)]
+			STATES[j,2]<-new
+			ii<-which(tt$edge[,1]==tt$edge[j,2])
+			if(length(ii)>0) STATES[ii,1]<-new
+		}
+		x<-as.factor(
+			setNames(sapply(1:Ntip(tt),function(n,S,E) S[which(E==n)],
+			S=STATES[,2],E=tt$edge[,2]),tt$tip.label))
+		if(nsim>1) X[[i]]<-x else X<-x
+	}
+	X
+}
